type cmp =
  | Less
  | Greater
  | Equal

type ('a, 'b) part =
  | Left of 'a
  | Right of 'b
  | Discard

val map_fold : ('a -> 'b) -> 'a list -> 'b list
val map_rec : ('a -> 'b) -> 'a list -> 'b list

val partition_fold : ('a -> ('b, 'c) part) -> 'a list -> 'b list * 'c list
val partition_rec : ('a -> ('b, 'c) part) -> 'a list -> 'b list * 'c list

val mean_fold : float list -> float
val mean_rec : float list -> float

val all_prefixes_fold : 'a list -> 'a list list
val all_prefixes_rec : 'a list -> 'a list list

val average_heading_fold : float list -> float
val average_heading_rec : float list -> float

module type N = sig
  (** [t] represents a natural number. *)
  type t

  (** [zero] returns the [t] representing 0. *)
  val zero : t

  (** [succ x] returns the successor to [x]. *)
  val succ : t -> t

  (** [eq a b] returns [a = b] (but you might implement it
      differently! *)
  val eq : t -> t -> bool
end

module IntNat : N
module PeanoNat : N

module type NAT_OPS = sig
  (* We expose that [t = PeanoNat.t] so that [PeanoNat.t]s
     can be used with the functions in this module. *)
  type t = PeanoNat.t

  (** [add a b] returns [a + b]. *)
  val add : t -> t -> t

  (** [sub a b] returns [a - b]. If [a < b], the result is
      undefined. *)
  val sub : t -> t -> t
    
  (** [cmp a b] returns [Less] if [a < b], [Greater] if
      [a > b], and [Equal] if [a = b]. *)
  val cmp : t -> t -> cmp

  (** [mul a b] returns [a * b]. *)
  val mul : t -> t -> t

  (** [div a b] returns [(q, r)] such that [a = bq + r]
      with [r < b]. *)
  val div : t -> t -> t * t
end

module NatOps : NAT_OPS

module type Q = sig
  type t

  (** [make p q] returns [Some x] where [x] represents the
      rational [p/q] if [q <> 0] and [None] otherwise. *)
  val make : int -> int -> t option

  (** [add a b] returns [a + b]. *)
  val add : t -> t -> t

  (** [inv x] returns [1/x]. *)
  val inv : t -> t

  (** [neg x] returns [-x]. *)
  val neg : t -> t

  (** [mul a b] returns [a * b]. *)
  val mul : t -> t -> t
  
  (** [eq a b] returns [a = b]. *)
  val eq : t -> t -> bool

  (** [cmp a b] returns [Less] if [a < b], [Greater] if
      [a > b], and [Equal] if [a = b]. *)
  val cmp : t -> t -> cmp

  (** [div a b] returns [a / b]. *)
  val div : t -> t -> t
  
  (** [gcd a b] returns the gcd of [a] and [b]. *)
  val gcd : t -> t -> t

  (** [to_int x] returns [p] and [q] such that [p/q = x]. *)
  val to_int : t -> int * int
end

module Rational : Q

module type SKIP_SET = sig
  (** An ['a t] represents a set containing elements of type
      ['a]. *)
  type 'a t

  (** [make p xs] creates a new [t] containing the elements of
      [xs]. [p] is the probability that an element in a leve
      of the skip set is present in the next level of the skip 
      set. *)
  val make : float -> 'a list -> 'a t

  (** [mem x set] returns [true] iff [x] is an element of
      [set]. *)
  val mem : 'a -> 'a t -> bool
end

module SkipSet : SKIP_SET
